package jalview.ext.archaeopteryx;

import jalview.analysis.TreeBuilder;
import jalview.datamodel.SequenceI;
import jalview.gui.Desktop;
import jalview.viewmodel.AlignmentViewport;

import java.awt.Dimension;
import java.util.Map;

import org.forester.archaeopteryx.Archaeopteryx;
import org.forester.archaeopteryx.MainFrame;
import org.forester.phylogeny.Phylogeny;
import org.forester.phylogeny.PhylogenyNode;

public final class ArchaeopteryxInit
{
  /**
   * This method should generally not be used as it does not bind the tree to
   * its alignment
   * 
   * @param aptxTrees
   * @return
   */
  public static MainFrame createUnboundInstance(Phylogeny aptxTree)
  {
    Phylogeny[] aptxTrees = { aptxTree };
    return createBoundAptxFrame(aptxTrees, null);
  }

  public static MainFrame createInstance(Phylogeny[] aptxTrees,
          AlignmentViewport jalviewAlignmentView)
  {
    return createBoundAptxFrame(aptxTrees, jalviewAlignmentView);

  }

  public static MainFrame createInstance(Phylogeny aptxTree,
          AlignmentViewport jalviewAlignmentView)
  {
    Phylogeny[] aptxTrees = { aptxTree }; // future possibility to load in
    // several trees simultaneously
    return createBoundAptxFrame(aptxTrees, jalviewAlignmentView);

  }

  public static MainFrame createInstance(
          TreeBuilder calculatedTree) // very dense method, to be split up
  {
    ArchaeopteryxTreeConverter aptxTreeBuilder = new ArchaeopteryxTreeConverter(
            calculatedTree);

    Phylogeny aptxTree = aptxTreeBuilder.buildAptxTree();
    Phylogeny[] aptxTrees = { aptxTree }; // future possibility to load in
                                          // several trees simultaneously

    MainFrame aptxApp = createBoundAptxFrame(aptxTrees,
            calculatedTree.getAvport());
    bindNodesToJalviewSequences(aptxApp, calculatedTree.getAvport(),
            aptxTreeBuilder.getAlignmentBoundNodes());
    return bindFrameToJalview(aptxApp);

  }




  private static MainFrame createBoundAptxFrame(Phylogeny[] aptxTrees,
          AlignmentViewport jalviewAlignmentView)
  {
    MainFrame aptxApp = Archaeopteryx.createApplication(aptxTrees,
            "_aptx_jalview_configuration_file", null);

    return aptxApp;
  }

  private static void bindNodesToJalviewSequences(MainFrame aptxApp,
          AlignmentViewport jalviewAlignViewport,
          Map<SequenceI, PhylogenyNode> alignMappedToNodes)
  {
    new JalviewAptxBinding(aptxApp, jalviewAlignViewport,
            alignMappedToNodes);
  }


  private static MainFrame bindFrameToJalview(MainFrame aptxApp)
  {
    int width = 400;
    int height = 550;
    aptxApp.setMinimumSize(new Dimension(width, height));

    Desktop.addInternalFrame(aptxApp, "Archaeopteryx Tree View", true,
            width, height, true, true);

    return aptxApp;

  }


}
